@@ -3,8 +3,8 @@
 //!
 //! * [x] [Stable storage](storage::StableStorage).
 //! * [x] [Append-only log](log::Log) using stable storage.
+//! * [x] [Flat merkle tree](flat_tree::FlatTree) that is well suited for stable storage.
 //! * [ ] Even queue.
-//! * [ ] Flat merkle tree.
 //!
 //! All source code are original and released under GPLv3.
 //! Please make sure you understand the requirement and risk before using them in your own projects.
@@ -12,5 +12,6 @@
 //! [Internet Computer]: https://wiki.internetcomputer.org
 //! [Spinner.Cash]: https://github.com/spinner-cash
 
+pub mod flat_tree;
 pub mod log;
 pub mod storage;
